@@ -1,10 +1,11 @@
-from .tm_elements import Drift, Dipole, Quadrupole, Sextupole
+from .tm_elements import Drift, Dipole, Quadrupole, Sextupole, TMElement
 from .plasma_stage import PlasmaStage
 from .plasma_ramp import PlasmaRamp
 from .active_plasma_lens import ActivePlasmaLens
 from .beamline import Beamline
+from .field_element import FieldElement
 
 
 __all__ = [
     'Drift', 'Dipole', 'Quadrupole', 'Sextupole', 'PlasmaStage', 'PlasmaRamp',
-    'ActivePlasmaLens', 'Beamline']
+    'ActivePlasmaLens', 'Beamline', 'FieldElement', 'TMElement']
